
import numpy as np
import os
import glob
import yaml


def FilePattern(var, levtype, freq, yaml_content, rundir, start_date, end_date):
   start_date_str = str(start_date)
   end_date_str = str(end_date)
   model = yaml_content['model']
   project = yaml_content['project']
   simulation = yaml_content['simulation']
   ensemble = yaml_content['ensemble']
   
   file_pattern = levtype + "_" + freq + "__" + project + "_" + model + "_" + simulation + "_" + ensemble + "_" + var + "_" + start_date_str + "-" + end_date_str
   file_pattern = rundir + "*" + file_pattern


   return file_pattern

def Construct_Dates(start_date, end_date, frequency, rundir):
    # DATE: YYYYMMDDZZ # 
    rundir = rundir + "/02-InterpolatedGrid"
    date_array = []
    if frequency == "DAILY":
        date_array = np.arange(start_date, end_date, 100)
        rundir = rundir + "/03-Daily"
    elif frequency == "6HOURLY":  # CAREFUL, WHEN REACHING 24 IT GOES TO 30
        start_date = str(start_date)
        end_date = str(end_date)
        start_date_date = start_date[:8]
        start_date_time = start_date[8:]
        start_date_reformat = f"{start_date_date[:4]}-{start_date_date[4:6]}-{start_date_date[6:]} {int(start_date_time):02d}:00:00"

        end_date_date = end_date[:8]
        end_date_time = end_date[8:]
        end_date_reformat = f"{end_date_date[:4]}-{end_date_date[4:6]}-{end_date_date[6:]} {int(end_date_time):02d}:00:00"
        date_array = pd.date_range(start=start_date_reformat, end=end_date_reformat, freq='6H',)
        date_array = date_array.strftime('%Y%m%d%H')
        date_array = date_array.to_numpy()
        rundir = rundir + "/02-6Hourly"

    elif frequency == "MONTHLY":
        date_array = np.arange(start_date, end_date, 10000)
        rundir = rundir + "/04-Monthly"
    else:
        print("ERROR, no correct frequency for date reconstruction")
        return []
    return date_array, rundir

### Refactoring code to improve readability and logic ###
def check_file(pattern):
    #print(f"Searching for pattern '{pattern}'")
    matching_files = glob.glob(pattern)
    
    
    if matching_files:
        found = matching_files
    else:
        missing = pattern
        found = ""
        print(f"No files matched the pattern '{pattern}'")

    return found, missing
        
    
def process_variable(var, levtype, freq, content, rundir, levels):
    start_date = yaml_content['start_date']       ## Careful when we are getting this info
    end_date = yaml_content['end_date']
    dates, rundir = Construct_Dates(start_date, end_date, freq, rundir)
    print(f"Rundir is '{rundir}'")
    file_pattern = FilePattern(var, levtype, freq, yaml_content, rundir, start_date, end_date)
    d_req_missing = []
    for date in dates:
        date_pattern = file_pattern + "-" + str(date)
        for level in levels:
            final_pattern = date_pattern + "_level_" + str(level) + ".nc"
            found, missing = check_file(final_pattern)
            if missing:
                d_req_missing.append(missing)

    print(f"Some files are missing for variable '{var}'")
    print(d_req_missing)

def process_levtype(levtype, freq, content, rundir, levels ):
    lev = levtype.get('levtype')
    print(f"Levtype is '{lev}'")

    # Process variables in each levtype

    for var in levtype.get('netcdf_name', []):
        print(f"Variable is '{var}'")
        process_variable(var, lev, freq, content, rundir, levels)


def process_frequency(freq, content, rundir, levels):
    print(f"Frequency is '{freq}'")
    levtypes = content.get(freq, [])
    # Process each levtype

    for levtype in levtypes:
        process_levtype(levtype, freq, content, rundir, levels)

levels = np.array([1000, 925, 850, 700, 600, 500, 400, 300, 250, 200, 150, 100, 70, 50, 30, 20, 10, 5, 1])
data_portfolio = "/home/maxness/Install_MultIO/sources/PostProcessing/Data-Portfolio-EERIE.yaml"
rundir =  "/home/maxness/delete"
with open(data_portfolio, 'r') as file:
    yaml_content = yaml.safe_load(file)

content = yaml_content.get("test", {})
for freq in content:
    process_frequency(freq, content, rundir, levels)